window.Gleam.initSearch([{"doc":"gxid","title":"gxid","content":"# gxid\n\n![CI](https://github.com/defgenx/gxid/workflows/test/badge.svg?branch=master)\n[![Package Version](https://img.shields.io/hexpm/v/gxid)](https://hex.pm/packages/gxid)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/gxid/)\n\nA Gleam implementation of xid\n\n## Quick start\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\ngleam shell # Run an Erlang shell\n```\n\n## Installation\n\nThis package can be added to your Gleam project:\n\n```sh\ngleam add gxid\n```\n\nand its documentation can be found at &lt;https://hexdocs.pm/gxid&gt;.\n\n## Usage\n\n### Generate a new XID\n\n```gleam\nimport gxid.{type XID}\n\nlet assert Ok(channel) = gxid.start()\n\nlet xid: XID = gxid.generate(channel)\nlet str_xid = xid\n              |&gt; gxid.string()\n```\n\n### From an existing XID\n\n```gleam\nimport gxid.{type XID}\n\nlet xid: XID = gxid.parse(&quot;h8a8u4o00de6hq6tsc00&quot;)\nlet str_xid = xid\n              |&gt; gxid.string()\n```\n\n## About XID\n\nXid uses the Mongo Object ID algorithm to generate globally unique ids with a different serialization (base64) to make\nit shorter when transported as a string:\nhttps://docs.mongodb.org/manual/reference/object-id/\n\n- 4-byte value representing the seconds since the Unix epoch,\n- 3-byte machine identifier,\n- 2-byte process id, and\n- 3-byte counter, starting with a random value.\n\nThe binary representation of the id is compatible with Mongo 12 bytes Object IDs.\nThe string representation is using base32 hex (w/o padding) for better space efficiency\nwhen stored in that form (20 bytes). The hex variant of base32 is used to retain the\nsortable property of the id.\n\nXid doesn&#39;t use base64 because case sensitivity and the 2 non alphanum chars may be an\nissue when transported as a string between various systems. Base36 wasn&#39;t retained either\nbecause 1/ it&#39;s not standard 2/ the resulting size is not predictable (not bit aligned)\nand 3/ it would not remain sortable. To validate a base32 `xid`, expect a 20 chars long,\nall lowercase sequence of `a` to `v` letters and `0` to `9` numbers (`[0-9a-v]{20}`).\n\nUUIDs are 16 bytes (128 bits) and 36 chars as string representation. Twitter Snowflake\nids are 8 bytes (64 bits) but require machine/data-center configuration and/or central\ngenerator servers. xid stands in between with 12 bytes (96 bits) and a more compact\nURL-safe string representation (20 chars). No configuration or central generator server\nis required so it can be used directly in server&#39;s code.\n\n| Name        | Binary Size | String Size    | Features\n|-------------|-------------|----------------|----------------\n| [UUID]      | 16 bytes    | 36 chars       | configuration free, not sortable\n| [shortuuid] | 16 bytes    | 22 chars       | configuration free, not sortable\n| [Snowflake] | 8 bytes     | up to 20 chars | needs machine/DC configuration, needs central server, sortable\n| [MongoID]   | 12 bytes    | 24 chars       | configuration free, sortable\n| xid         | 12 bytes    | 20 chars       | configuration free, sortable\n\n[UUID]: https://en.wikipedia.org/wiki/Universally_unique_identifier\n\n[shortuuid]: https://github.com/stochastic-technologies/shortuuid\n\n[Snowflake]: https://blog.twitter.com/2010/announcing-snowflake\n\n[MongoID]: https://docs.mongodb.org/manual/reference/object-id/\n\nReferences:\n\n- http://www.slideshare.net/davegardnerisme/unique-id-generation-in-distributed-systems\n- https://en.wikipedia.org/wiki/Universally_unique_identifier\n- https://blog.twitter.com/2010/announcing-snowflake\n- Go port by [Olivier Poitrey](https://github.com/rs): https://github.com/rs/xid\n- Python port by [Graham Abbott](https://github.com/graham): https://github.com/graham/python_xid\n- Scala port by [Egor Kolotaev](https://github.com/kolotaev): https://github.com/kolotaev/ride\n- Rust port by [Jérôme Renard](https://github.com/jeromer/): https://github.com/jeromer/libxid\n- Ruby port by [Valar](https://github.com/valarpirai/): https://github.com/valarpirai/ruby_xid\n- Java port by [0xShamil](https://github.com/0xShamil/): https://github.com/0xShamil/java-xid\n- Dart port by [Peter Bwire](https://github.com/pitabwire): https://pub.dev/packages/xid\n- PostgreSQL port by [Rasmus Holm](https://github.com/crholm): https://github.com/modfin/pg-xid\n- Swift port by [Uditha Atukorala](https://github.com/uditha-atukorala): https://github.com/uditha-atukorala/swift-xid\n- C++ port by [Uditha Atukorala](https://github.com/uditha-atukorala): https://github.com/uditha-atukorala/libxid\n\n\n","url":"index.html"},{"doc":"gxid","title":"Message","content":"pub opaque type Message\n A library implementing XID generation\n Message handled by the actor\n\nSynonyms:\ngxid.Message\ngxid Message","url":"gxid.html#Message"},{"doc":"gxid","title":"State","content":"pub opaque type State\n Actor&#39;s State internal representation\n\nSynonyms:\ngxid.State\ngxid State","url":"gxid.html#State"},{"doc":"gxid","title":"XID","content":"pub opaque type XID\n XID representation\n\n This is an opaque type to ensure its RO nature\n\nSynonyms:\ngxid.XID\ngxid XID","url":"gxid.html#XID"},{"doc":"gxid","title":"generate","content":"pub fn generate(channel: Subject(Message)) -&gt; XID\n Generates a XID\n\n ### Usage\n \n import gxid.{XID}\n\n let assert Ok(channel) = gxid.start()\n\n let xid: XID = gxid.generate(channel)\n \n See: https:///hexdocs.pm/gleam_otp/0.1.1/gleam/otp/actor/#call\nSynonyms:\ngxid.generate\ngxid generate","url":"gxid.html#generate"},{"doc":"gxid","title":"machine_id","content":"pub fn machine_id(xid: XID) -&gt; Int\n Returns machine id from XID type\nSynonyms:\ngxid.machine_id\ngxid machine_id","url":"gxid.html#machine_id"},{"doc":"gxid","title":"parse","content":"pub fn parse(str_xid: String) -&gt; XID\n Parse a XID string to have its composition\n\n ### Usage\n \n import gxid.{XID}\n\n let xid: XID = gxid.parse(&quot;h8a8u4o00de6hq6tsc00&quot;)\n \nSynonyms:\ngxid.parse\ngxid parse","url":"gxid.html#parse"},{"doc":"gxid","title":"pid","content":"pub fn pid(xid: XID) -&gt; Int\n Returns PID from XID type\nSynonyms:\ngxid.pid\ngxid pid","url":"gxid.html#pid"},{"doc":"gxid","title":"random_number","content":"pub fn random_number(xid: XID) -&gt; Int\n Returns random number from XID type\nSynonyms:\ngxid.random_number\ngxid random_number","url":"gxid.html#random_number"},{"doc":"gxid","title":"start","content":"pub fn start() -&gt; Result(Subject(Message), StartError)\n Starts State generator\nSynonyms:\ngxid.start\ngxid start","url":"gxid.html#start"},{"doc":"gxid","title":"string","content":"pub fn string(xid: XID) -&gt; String\n Returns xid string representation from XID type\nSynonyms:\ngxid.string\ngxid string","url":"gxid.html#string"},{"doc":"gxid","title":"time","content":"pub fn time(xid: XID) -&gt; Int\n Returns time from XID type\nSynonyms:\ngxid.time\ngxid time","url":"gxid.html#time"},{"doc":"gxid","title":"to_xid","content":"pub fn to_xid(bit_xid: BitArray) -&gt; XID\n Copy a BitArray to a new XID and encode\nSynonyms:\ngxid.to_xid\ngxid to_xid","url":"gxid.html#to_xid"},{"doc":"gxid","title":"gxid","content":"","url":"gxid.html"}]);